# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package XJDParser;

use strict;
use Data::Dumper;
use XML::Simple;

sub new {
    my $classname  = shift;         # What class are we constructing?
    my $rcfilename = shift;
    my $self   = {};                # Allocate new memory
    bless($self, $classname);       # Mark it of the right type
    $self->_init($rcfilename);
    return $self;
}

# "private" method to initialize fields.
sub _init {
   my $self = shift;
   my $rcfile = shift;

   $self->{rc} = XMLin("$rcfile",
           forcearray=>['component', 'connection'],
           KeyAttr => "id",
           SuppressEmpty => '' );

   #Debug purpose
   #open O1, "> temp-iyoon" or die "I'm dying!\n";
   #my $logstr = Dumper($self->{rc});
   #print O1 "$logstr\n";
}

sub getComponentList(){
    my $self = shift;
    return $self->{rc}->{components}->{component};
}

sub findComponent($){
    my $self = shift;
    my $givenkey = shift;

    my $cList = $self->getComponentList();
    my $found = undef;

    foreach my $key (keys %{$cList}){
        if($key eq $givenkey){
            print STDERR "\tI found the component $key with name " . $cList->{$key}->{name} . "\n";
            return $cList->{$key};
        }
    }

    return $found;
}

sub printList($){
    my $self = shift;
    my $what = shift;

    print "======================= $what list =======================\n";
    my $list = $self->{rc}->{$what};
    foreach(keys %{$list}){
        printComponent($list->{$_});
    }
}

sub printComponent($){
    my $comp = shift;

    print "\t------------------------------------------------------\n";
    print "\t|  node     : ".$comp->{nNode}."\n";
    print "\t------------------------------------------------------\n";
}

sub getConnectionList(){
    my $self = shift;
    
    return $self->{rc}->{connections}->{connection};
}


1;
