#
# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package show_modify_detail;

use PerlDB;
use utilfunc;
use XJDParser;
use Data::Dumper;
use strict;

sub new {
    my $classname  = shift;         # What class are we constructing?
    my $self   = {};                # Allocate new memory
    bless($self, $classname);       # Mark it of the right type
    return $self;
}

#
# We can safely assume that 'sessionid'.xjd file exist 
# because user can reach this subroutine by pressing component or connection
# in the element list frame, which is shown by parsing the xjd file.
#
sub showContents {
    my $self = shift;
    my $cgi = shift;

    #&utilfunc::print_params($cgi);

    my $sessionid = &utilfunc::get_session_id ($cgi);
    my $DEBUGLEVEL = 2;
    my $prevXJD = new XJDParser("$ENV{HTDOCS_HOME}/HPCALE_XJDHELPER/sessions/$sessionid.xjd", $DEBUGLEVEL);

    my $data = &prepareData($cgi, $prevXJD);
    if($data == -1){
        print "Error in loading ".$cgi->param('id')."!";
        return;
    }
    
    if($cgi->param('eletype') eq "component"){
        &showComponents ($cgi, $data);    
    }
    else{
        &showConnections ($cgi, $data, $prevXJD);
    }
}

sub defCompHiddenCGIVar {
    my ($cgi, $data) = @_;

    print $cgi->hidden(-name=>'mode', -default=>"modify_detail");
    print $cgi->hidden(-name=>'eletype', -default=>"component");
    print $cgi->hidden(-name=>'id', -default=>$cgi->param('id'));
    print $cgi->hidden(-name=>'name', -default=>$data->{name});
    print $cgi->hidden(-name=>'type', -default=>$data->{type});
    
    if(defined $data->{rcid}){
        # This is the problematic part. 
        # If {rcid} is empty in XJD file, when it is parsed by our XMLin function,
        # $data->{rcid} will has empty hash. (not string)
        # We don't have a way to know the type of this variable.
        # When I try to assign this hash as a default value of following hidden 
        # function, program stops at this point.
        
        print $cgi->hidden(-name=>'rcid', -default=>$data->{rcid});
    }
    
    if(defined $data->{cluster}){
        print $cgi->hidden(-name=>'rctype', -default=>"cluster");
        print $cgi->hidden(-name=>'cluster', -default=>$data->{cluster});
        print $cgi->hidden(-name=>'scheduler', -default=>$data->{scheduler});
        print $cgi->hidden(-name=>'head', -default=>$data->{head});
    }

    if(defined $data->{node}){
        unless(defined $cgi->param('domain')){
            $data->{head} =~ /(\w+)\.(.+)/;
            $data->{node} = $1;
            $data->{domain} = $2;
        }
        print $cgi->hidden(-name=>'rctype', -default=>"node");
        print $cgi->hidden(-name=>'head', -default=>$data->{head});
        print $cgi->hidden(-name=>'node', -default=>$data->{node});
        print $cgi->hidden(-name=>'domain', -default=>$data->{domain});
    }

    print $cgi->hidden(-name=>'parallel', -default=>$data->{parallel});
    print $cgi->hidden(-name=>'path', -default=>$data->{path});
    print $cgi->hidden(-name=>'file', -default=>$data->{file});
}    
    
sub showComponents {
    my ($cgi, $data) = @_;
    
    print $cgi->start_form(-name=>'modify_detail',
                           -onsubmit=>"return validate_comp_modification(this)");
    
    &defCompHiddenCGIVar ($cgi, $data);

    print $cgi->start_table({-border => 1});
    
    &utilfunc::printSimpleRow ($cgi, $data, 1, "Element", "Value", "");
    &utilfunc::printSimpleRow ($cgi, $data, 1, "ID", $cgi->param('id'), "desc_id");
    &utilfunc::printSimpleRow ($cgi, $data, 0, "Name", "name", "desc_name");
    &utilfunc::printSimpleRow ($cgi, $data, 0, "Type", "type", "desc_type");

    if(defined $data->{cluster}){
        &utilfunc::printSimpleRow ($cgi, $data, 0, "Cluster", "cluster", "desc_cluster");
        &utilfunc::printSimpleTextField($cgi, $data, "Number of Nodes", "nNode", 50, 80, "desc_nNode");
        &utilfunc::printSimpleRow ($cgi, $data, 0, "Scheduler", "scheduler", "desc_sched");

        if($data->{scheduler} eq "SSH"){
            &utilfunc::printSimpleTextField($cgi, $data, "NodeSet", "nodeset", 50, 80, "desc_nodeset");
        }        
    }
    elsif(defined $data->{node}){
        &utilfunc::printSimpleRow ($cgi, $data, 0, "Nodename", "head", "desc_node");
    }

    if($data->{parallel} eq "1"){
        &utilfunc::printSimpleRow ($cgi, $data, 1, "Parallel", "Y", "desc_parallel");
    }
    else{
        &utilfunc::printSimpleRow ($cgi, $data, 1, "Parallel", "N", "desc_parallel");
    }
    
    &utilfunc::printSimpleTextField($cgi, $data, "Argument", "argument", 50, 255, "desc_argument");
    &utilfunc::printSimpleTextField($cgi, $data, "Launch Directory", "launchdir", 50, 255, "desc_launchdir");
    &utilfunc::printSimpleTextField($cgi, $data, "Launch Program", "launchprog", 50, 255, "desc_launchprog");
    &utilfunc::printSimpleTextField($cgi, $data, "Launch Argument", "launcharg", 50, 255, "desc_launcharg");
    &utilfunc::printSimpleTextField($cgi, $data, "Send Filelist", "sendfiles", 50, 80, "desc_sfilelist");
    &utilfunc::printSimpleTextField($cgi, $data, "Receive Filelist", "recvfiles", 50, 80, "desc_rfilelist");
 
    print $cgi->end_table();
    print "<br>";
    
    print $cgi->submit(-name=>'submit', -value=>'Commit Modification', -onClick=>"document.pressed=this.value"); 
    print $cgi->submit(-name=>'submit', -value=>'Remove Component', -onClick=>"document.pressed=this.value"); 

    print $cgi->end_form;
    print $cgi->hr;
}


sub showConnections {
    my ($cgi, $data, $prevXJD) = @_;

    my @carr = ();
    my %clabel=();
    my @exarr = ();
    my @imarr = ();

    &print_javascript ($prevXJD, 
                       $data->{exporter}, $data->{importer}, 
                       \@carr, \%clabel, \@exarr, \@imarr) ;

    print $cgi->start_form(-name=>'modify_detail',
                           -onsubmit=>"return validate_conn(this)");
    
    print $cgi->hidden(-name=>'mode', -default=>"modify_detail");
    print $cgi->hidden(-name=>'eletype', -default=>"connection");
    print $cgi->hidden(-name=>'connid', -default=>$cgi->param('id'));

    print $cgi->start_table({-border => 1});

    &utilfunc::printSimpleRow ($cgi, $data, 1, "Connection ID",  $cgi->param('id'), "desc_connid");
    
    &showExporter ($cgi, $data, \@carr, \%clabel, \@exarr);
    &showImporter ($cgi, $data, \@carr, \%clabel, \@imarr);

    # message tag
    &utilfunc::printSimpleTextField($cgi, $data, "Message Tag", "msgtag", 20, 20, "desc_msgtag");

    # data type
    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});
    print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_datatype.html", 
                            -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_datatype.html')"},
                           "Data Type"));
    print $cgi->td($cgi->popup_menu(-name=>'datatype',
                                    -values=>['int','short','char','float','double'],
                                    #-default=>'int'));
                                    -default=>"$data->{type}"));
    print $cgi->end_Tr();

   # comm type
    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});
    print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_commtype.html", 
                            -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_commtype.html')"},
                           "Communication Type"));
    print $cgi->td($cgi->popup_menu(-name=>'commtype',
                                    -values=>['MxN','1xN'],
                                    #-default=>'MxN'));
                                    -default=>"$data->{commtype}"));
    print $cgi->end_Tr();

   # end table
    print $cgi->end_table();
    print "<br>";

    print $cgi->submit(-name=>'submit', -value=>'Commit Modification', -onClick=>"document.pressed=this.value"); 
    print $cgi->submit(-name=>'submit', -value=>'Remove Connection', -onClick=>"document.pressed=this.value"); 
    
    print $cgi->end_form;          
	print $cgi->hr;
}

sub showExporter {
    my ($cgi, $data, $refcarr, $refclabel, $refexarr) = @_;
    
    # Exporter
    my $i=0;
    my $default_exporter = 0;
    foreach(@{$refcarr}){
        if($_ eq $data->{exporter}){
            $default_exporter = $i;
        }
        $i++;
    }
    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});
    print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_exporter.html", 
                            -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_exporter.html')"},
                           "Exporter"));
    print $cgi->td($cgi->popup_menu(-name=>'excomp',
                                    -values=>[@{$refcarr}],
                                    -default=>$refcarr->[$default_exporter],
                                    -labels=>$refclabel,
                                    -onchange=>"change_exporter(this.form)",
                                    -onselect=>"change_exporter(this.form)"));
    print $cgi->end_Tr();

    # Exporter Ports
    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});
    print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_expport.html", 
                            -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_expport.html')"},
                           "Export Port"));
    print $cgi->td($cgi->popup_menu(-name=>'exdataport',
                                    -values=>[@{$refexarr}]));
    print $cgi->end_Tr();

}


sub showImporter {
    my ($cgi, $data, $refcarr, $refclabel, $refimarr) = @_;
    
    # Importer
    my $i=0;
    my $default_importer = 0;
    foreach(@{$refcarr}){
        if($_ eq $data->{importer}){
            $default_importer = $i;
        }
        $i++;
    }
    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});
    print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_importer.html", 
                            -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_importer.html')"},
                           "Importer"));
    print $cgi->td($cgi->popup_menu(-name=>'imcomp',
                                    -values=>[@{$refcarr}],
                                    -default=>$refcarr->[$default_importer],
                                    -labels=>$refclabel,
                                    -onchange=>"change_importer(this.form)",
                                    -onselect=>"change_importer(this.form)"));
    print $cgi->end_Tr();

    # Importer Ports
    my $j = 0;
    my $default_import_port = 0;
    foreach(@{$refimarr}){
        if($_ eq $data->{importport}){
            $default_import_port = $j;
        }
        $j++;
    }
    
    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});
    print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_impport.html", 
                            -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_impport.html')"},
                           "Import Port"));
    print $cgi->td($cgi->popup_menu(-name=>'imdataport',
                                    -values=>[@{$refimarr}],
                                    -default=>$refimarr->[$default_import_port]));
    print $cgi->end_Tr();

}

sub print_javascript {
    my ($prevXJD, $iniExp, $iniImp, $refcarr, $refclabel, $refexarr, $refimarr) = @_;
    
    my @comparr = ();   
    while ( my ($k, $v) = each(%{$prevXJD->{rc}->{components}->{component}}) ) {
        push @comparr, [$k, $v->{name}, $v->{rcid}];
    }

    my @exdataport = ();
    my @imdataport = ();

    my $perldb = new PerlDB();

    foreach(@comparr){
        my $uniqueid = $_->[0];
        my $name = $_->[1];
        my $rc_id = $_->[2];

        my $query = "SELECT id FROM component WHERE compID = '$name' AND rc_id = $rc_id";
        my $cid = $perldb->select($query)->[0];

        next if(! defined $cid);
        next if(length($cid) == 0);

        $query = "SELECT port_name, port_type FROM dataport WHERE component_id = $cid";
        my $compref = $perldb->multcol_select($query);

        next if(not defined $compref);
        my @comparray = @{$compref};

        foreach(@comparray){
            my $pname = $_->[0];
            my $ptype = $_->[1];

            if($ptype eq "export"){
                push @exdataport, [$uniqueid, $pname];
            }
            elsif($ptype eq "import"){
                push @imdataport, [$uniqueid, $pname];
            }

        }
    }
    
    print "<script type=\"text/javascript\" language=\"javascript\">\n";
    print "<!--\n";
    print "var complist = Array(".scalar @comparr.");\n";
    print "var exportlist = Array(".scalar @exdataport.");\n";
    print "var exportcomplist = Array(".scalar @exdataport.");\n";
    print "var importlist = Array(".scalar @imdataport.");\n";
    print "var importcomplist = Array(".scalar @imdataport.");\n";
     
    my $i=0;
    foreach(@comparr){
        print "complist[$i] = \"$_->[0]\"\n";
        $i++; 
    }
    $i=0;
    foreach(@exdataport){
        print "exportlist[$i] = \"$_->[1]\";\n";
        print "exportcomplist[$i] = \"$_->[0]\";\n";
        $i++;
    }
    $i=0;
    foreach(@imdataport){
        print "importlist[$i] = \"$_->[1]\";\n";
        print "importcomplist[$i] = \"$_->[0]\";\n";
        $i++;
    }

    &write_updatefunctions ;
    
    print "//-->\n";
    print "</script>\n";
    
    $perldb->db_end ;


    # For display in popup-menu
    foreach(@comparr){
        push @{$refcarr}, $_->[0];
        $refclabel->{$_->[0]} = "$_->[0] : $_->[1]";
    }

    # For initial exporter display in popup-menu
    $i = 0;
    my $default_exporter = 0;
    foreach(@{$refcarr}){
        if($_ eq $iniExp){
            $default_exporter = $i;
        }
        $i++;
    }
    foreach(@exdataport){
        if($_->[0] eq $comparr[$default_exporter]->[0]){
            push @{$refexarr}, $_->[1];
        }
    }

    # For initial importer display in popup-menu
    my $j = 0;
    my $default_importer = 0;
    foreach(@{$refcarr}){
        if($_ eq $iniImp){
            $default_importer = $j;
        }
        $j++;
    }
    foreach(@imdataport){
        if($_->[0] eq $comparr[$default_importer]->[0]){
            push @{$refimarr}, $_->[1];
        }
    }
}


sub write_updatefunctions {
print<<UPDATEFUNCTION; 

    function change_exporter (f){
        var value;
        
        for ( var i = 0; i < f.excomp.options.length; ++i ) {
            if( f.excomp.options[i].selected ) {
                value = f.excomp.options[i].value;
            }
        }
    
        var n = 0;
        f.exdataport.options.length = 0;
        for( var i = 0 ; i < exportcomplist.length ; ++i) {
            if(exportcomplist[i] == value){
                f.exdataport.options[n] = new Option(exportlist[i]);
                ++n;
            }    
        }
        f.exdataport.options[0].selected = true;
    }
        
    function change_importer (f){
        var value;
        
        for ( var i = 0; i < f.imcomp.options.length; ++i ) {
            if( f.imcomp.options[i].selected ) {
                value = f.imcomp.options[i].value;
            }
        }
    
        var n = 0;
        f.imdataport.options.length = 0;
        for( var i = 0 ; i < importcomplist.length ; ++i) {
            if(importcomplist[i] == value){
                f.imdataport.options[n] = new Option(importlist[i]);
                ++n;
            }    
        }
        f.imdataport.options[0].selected = true;
    }

UPDATEFUNCTION
}

sub prepareData {
    my $cgi = shift;
    my $prevXJD = shift;
    
    my $id = $cgi->param('id');
    my $eletype = $cgi->param('eletype');
 
    if($eletype eq "component"){
        print $cgi->h1("Modify Component Detail.");
        return &collectComponentData ($cgi, $id, $prevXJD);
    }
    else{
        print $cgi->h1("Modify Connection Detail.");
        return &collectConnectionData ($cgi, $id, $prevXJD);
    }    
}    

sub collectComponentData {
    my ($cgi, $id, $prevXJD) = @_;

    while ( my ($k, $v) = each(%{$prevXJD->{rc}->{components}->{component}}) ) {
        if($k eq $id){
            #print "$k is found in XJD";
            return $v;
        }
    }
    
    print "Critical Error! Component $id is not found in XJD.\n";
}    

sub collectConnectionData {
    my ($cgi, $id, $prevXJD) = @_;

    while ( my ($k, $v) = each(%{$prevXJD->{rc}->{connections}->{connection}}) ) {
        if($k eq $id){
            #print "$k is found in XJD";
            return $v;
        }
    }

    print "Critical Error! Connection $id is not found in XJD.\n";
}    
    
1;
