#
# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package utilfunc;

use PerlDB;
use XJDParser;
use Data::Dumper;
use strict;

sub get_session_id {
    my $cgi = shift;
    return $cgi->cookie('sessionID');
}

sub save_comp {
    my $cgi = shift;
    my $sessionid = &get_session_id($cgi);

    print STDERR "save_comp for the session $sessionid\n";

    my $xjd = ();
    $xjd->{ICXJD} = ();
    $xjd->{ICXJD}->{version} = "1.5";
    $xjd->{ICXJD}->{components} = ();
    $xjd->{ICXJD}->{components}->{component} = ();
    $xjd->{ICXJD}->{components}->{component}->{component} = ();
    
    if(-e "$ENV{HTDOCS_HOME}/HPCALE_XJDHELPER/sessions/$sessionid.xjd"){
        my $DEBUGLEVEL = 2;
        my $prevXJD = new XJDParser("$ENV{HTDOCS_HOME}/HPCALE_XJDHELPER/sessions/$sessionid.xjd", $DEBUGLEVEL);

        # preserve the connection information    
        if(defined $prevXJD->{rc}->{connections}){
            $xjd->{ICXJD}->{connections} = $prevXJD->{rc}->{connections};
        }

        my $aComp = &gen_newcomp ($cgi);

        unless($prevXJD->{rc}->{components} eq ''){
            while ( my ($k, $v) = each(%{$prevXJD->{rc}->{components}->{component}}) ) {
                $xjd->{ICXJD}->{components}->{component}->{component}->{$k} = $v;
            }
        }

        # same key component will be replaced.
        if(keys (%{$aComp}) > 0){
            $xjd->{ICXJD}->{components}->{component}->{component}->{$cgi->param('id')} = $aComp;
            (new XJDWriter())->writeXML($xjd, "$ENV{HTDOCS_HOME}/HPCALE_XJDHELPER/sessions/$sessionid.xjd");
        }
        else{
            print STDERR "fail to save component ".$cgi->param('id');
            return -1;
        }
    }
    else{
        `touch $ENV{HTDOCS_HOME}/HPCALE_XJDHELPER/sessions/$sessionid.xjd`;

        my $aComp = &gen_newcomp ($cgi);
        if(keys (%{$aComp}) > 0){
            $xjd->{ICXJD}->{components}->{component}->{component}->{$cgi->param('id')} = $aComp;
            (new XJDWriter())->writeXML($xjd, "$ENV{HTDOCS_HOME}/HPCALE_XJDHELPER/sessions/$sessionid.xjd");
        }
        else{
            print STDERR "fail to save component ".$cgi->param('id');
            return -1;
        }
    }

    return 1;
}

sub gen_newcomp {
    my $cgi = shift;
    
    my %newcomp = ();
    
    $newcomp{'id'} = $cgi->param('id');
    $newcomp{'name'} = $cgi->param('name');    
    
    if($cgi->param('rctype') eq "cluster"){
        $newcomp{'cluster'} = $cgi->param('cluster');
        $newcomp{'scheduler'} = $cgi->param('scheduler');
        $newcomp{'head'} = $cgi->param('head');

        if($cgi->param('scheduler') eq "SSH"){
            $newcomp{'nodeset'} = $cgi->param('nodeset');
        }
        $newcomp{'nNode'} = $cgi->param('nNode');
    }
    elsif($cgi->param('rctype') eq "node"){
        # if this is invoked from show_detail.pm, {node} and {domain} is contained
        # however, if invoked from show_modify_detail.pm or show_mannew.pm, 
        # only {head} is included for convenience. (user doesn't need to specify those two 
        # separately. 
        # Here, we discriminate those two by checking existence of {domain} information.

        if(defined $cgi->param('domain')){
            $newcomp{'node'} = $cgi->param('node');
            $newcomp{'domain'} = $cgi->param('domain');
            $newcomp{'head'} = $cgi->param('head');
        }
        else{
            $newcomp{'head'} = $cgi->param('head');
            
            $newcomp{'head'} =~ /(\w+)\.(.+)/;
            $newcomp{'node'} = $1;

            if(defined $2){
                $newcomp{'domain'} = $2;
            }
            else{
                $newcomp{'domain'} = "";
            }
            
            if(length($newcomp{'node'}) == 0 || length($newcomp{'domain'}) == 0){
                my %emptyhash = ();

                print STDERR "Fail to store component, node name or domain is null";
                return \%emptyhash;
            }
        }    
    }

    $newcomp{'argument'} = $cgi->param('argument');
    $newcomp{'launchdir'} = $cgi->param('launchdir');
    $newcomp{'launcharg'} = $cgi->param('launcharg');
    $newcomp{'launchprog'} = $cgi->param('launchprog');
 
    if($cgi->param('parallel') eq "Y" || $cgi->param('parallel') == 1){
        $newcomp{'parallel'} = 1;
    }
    else{
        $newcomp{'parallel'} = 0;
    }
    $newcomp{'path'} = $cgi->param('path');
    $newcomp{'file'} = $cgi->param('file');
    
    # when manual 
    if(! defined $cgi->param('rcid')){
        $newcomp{'rcid'} = -1;

        my $perldb = new PerlDB();
        
        if($cgi->param('rctype') eq "cluster"){
            my $query1 = "SELECT rc_id FROM cluster WHERE clst_name = '$newcomp{'cluster'}'";
            my $var1 = $perldb->select($query1);
            if(scalar @{$var1} == 1){
                $newcomp{'rcid'} = $var1->[0];
            }
            else{
                my %emptyhash = ();
                print STDERR "Unknown Resource\n";
                return \%emptyhash;
            }
        }
        elsif ($cgi->param('rctype') eq "node"){
            my $query2 = "SELECT rc_id FROM node WHERE node_name = '$newcomp{'node'}' and domain = '$newcomp{'domain'}' ";
            my $var2 = $perldb->select($query2);
            if(scalar @{$var2} == 1){
                $newcomp{'rcid'} = $var2->[0];
            }
            else{
                my %emptyhash = ();
                print STDERR "Unknown Resource\n";
                return \%emptyhash;
            }
        }
        else{
            my %emptyhash = ();
            print STDERR "Unsupported Resource Type\n";
            return \%emptyhash;
        }
        
        $perldb->db_end ;
    }
    # when semi-automatic
    else{
        $newcomp{'rcid'} = $cgi->param('rcid');
    }
    $newcomp{'recvfiles'} = $cgi->param('recvfiles');
    $newcomp{'sendfiles'} = $cgi->param('sendfiles');
    $newcomp{'type'} = $cgi->param('type');

    return \%newcomp;
}

sub init_comp_params {
    my $cgi = shift;
    $cgi->delete('id','nNode','nodeset','compdata','argument', 'launchdir',
                 'launchprog','launcharg','sendfiles','recvfiles');
}


sub save_conn {
    my $cgi = shift;
    
    my $sessionid = &get_session_id($cgi);

    my $DEBUGLEVEL = 2;
    my $prevXJD = new XJDParser("$ENV{HTDOCS_HOME}/HPCALE_XJDHELPER/sessions/$sessionid.xjd", $DEBUGLEVEL);
    
    my $xjd = ();
    $xjd->{ICXJD} = ();

    # preserve the component information    
    $xjd->{ICXJD}->{components} = $prevXJD->{rc}->{components};

    $xjd->{ICXJD}->{connections} = ();
    $xjd->{ICXJD}->{connections}->{connection} = ();
    $xjd->{ICXJD}->{connections}->{connection}->{connection} = ();

    unless($prevXJD->{rc}->{connections} eq ''){
        while ( my ($k, $v) = each(%{$prevXJD->{rc}->{connections}->{connection}}) ) {
            $xjd->{ICXJD}->{connections}->{connection}->{connection}->{$k} = $v;
        }
    }
    
    my $aConn = &gen_newconn ($cgi) ;
    $xjd->{ICXJD}->{connections}->{connection}->{connection}->{$cgi->param('connid')} = $aConn;

    (new XJDWriter())->writeXML($xjd, "$ENV{HTDOCS_HOME}/HPCALE_XJDHELPER/sessions/$sessionid.xjd");
}

sub gen_newconn {
    my $cgi = shift;
    my %newconn = ();

    $newconn{'id'} = $cgi->param('connid');
    $newconn{'type'} = $cgi->param('datatype');
    $newconn{'commtype'} = $cgi->param('commtype');
    $newconn{'msgtag'} = $cgi->param('msgtag');
    $newconn{'exporter'} = $cgi->param('excomp');
    $newconn{'exportport'} = $cgi->param('exdataport');
    $newconn{'importer'} = $cgi->param('imcomp');
    $newconn{'importport'} = $cgi->param('imdataport');

    return \%newconn;
}

sub init_conn_params {
    my $cgi = shift;
    $cgi->delete('connid','datatype','msgtag','excomp','exdataport','imcomp','imdataport');
}


sub create_timestamp {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    
    my $myyear = $year + 1900;
    my $mymon = $mon+1;
    my $myday = $mday;
    
    return "$myyear-$mymon-$myday-$hour-$min-$sec";
}


sub print_params {
    my $cgi = shift;

    print "<hl>\n";
    print "CGI Parameter List <br>\n";
    print "<ul>\n";
    foreach($cgi->param){
        my $value = $cgi->param($_);
        print "<li>$_ : $value<br>\n";
    }
    print "</ul>\n";
    print "<hl>\n";
    print "<br>";
    print "Cookie List <br>\n";
    print "<ul>\n";
    if(defined $cgi->cookie('sessionID')){
        print "<li>".$cgi->cookie('sessionID')."<br>\n";
    }
    print "</ul>\n";
}

sub printEmptyRow {
    my ($cgi, $asis, $title, $field, $desc) = @_;

    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});

    if(length($desc) > 0 ){
        print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/$desc.html", 
                                -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/$desc.html')"},
                               "$title"));
        if($asis == 0){                               
            print $cgi->td("");
        }
        else{
            print $cgi->td($field);
        }
    }
    else{
        print $cgi->td($title);
        print $cgi->td($field);
    }
    
    print $cgi->end_Tr();
}

sub printSimpleRow {
    my ($cgi, $data, $asis, $title, $field, $desc) = @_;

    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});

    if(length($desc) > 0 ){
        print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/$desc.html", 
                                -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/$desc.html')"},
                               "$title"));
        if($asis == 0){                               
            print $cgi->td($data->{$field});
        }
        else{
            print $cgi->td($field);
        }
    }
    else{
        print $cgi->td($title);
        print $cgi->td($field);
    }
    
    print $cgi->end_Tr();
}

sub printEmptyTextField {
   my ($cgi, $title, $field, $size, $maxlen, $desc) = @_;

    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});

    if(length($desc) > 0 ){
        print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/$desc.html", 
                                -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/$desc.html')"},
                               "$title"));

        print $cgi->td($cgi->textfield( -name=>$field,
                                        -value=>"",
                                        -size=>$size,
                                        -maxlength=>$maxlen));
    }
    else{
        print $cgi->td($title);
        print $cgi->td($field);
    }
    
    print $cgi->end_Tr();
}


sub printSimpleTextField {
   my ($cgi, $data, $title, $field, $size, $maxlen, $desc) = @_;

    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});

    if(length($desc) > 0 ){
        print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/$desc.html", 
                                -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/$desc.html')"},
                               "$title"));

        my $value = $data->{$field};

        # 'suppressEmpty' option during XMLin function call, is used to set the 
        # empty value of elements as empty string.
        if($value eq "" || $value eq ''){
            $value = "";
        }

        print $cgi->td($cgi->textfield( -name=>$field,
                                        -value=>$value,
                                        -size=>$size,
                                        -maxlength=>$maxlen));
    }
    else{
        print $cgi->td($title);
        print $cgi->td($field);
    }
    
    print $cgi->end_Tr();
}


sub showTextFieldWithValue {
    my ($cgi, $descname, $name, $cginame, $value) = @_;
    
    print $cgi->start_Tr({-align=>'left', -valign=>'middle'});
    print $cgi->td($cgi->a({-href=>"https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_$descname.html", 
                            -onClick=>"return popup('https://$ENV{APACHE_SERVER}:$ENV{APACHE_PORT}/HPCALE_XJDHELPER/desc/desc_$descname.html')"},
                           $name));
    print $cgi->td($cgi->textfield( -name=>$cginame,
                                    -value=>$value,
                                    -size=>50,
                                    -maxlength=>80));
    print $cgi->end_Tr();
}

# Remove whitespace from the start and end of the string
sub trimwhitespace($)
{
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}


1;

